module rgb_inf(
    clk_in  ,           
    rst_n   ,
    dclk    ,
    hsync   ,
    vsync   ,
    //lcd_de  ,           //SYNC mode, lcd_de = 0;  
    lcd_dis ,           //lcd_backlight_en, HE;  
    data_r  ,
    data_g  ,
    data_b  
    );

    //
    parameter DATA_W    = 8     ;

    parameter THPW      = 20    ;   //unit:clk
    parameter THB       = 46    ;   //unit:clk
    parameter THD       = 800   ;   //unit:clk
    parameter THFP      = 210   ;   //unit:clk
    
    parameter TVPW      = 10    ;   //unit:line
    parameter TVB       = 23    ;   //unit:line
    parameter TVD       = 480   ;   //unit:line
    parameter TVFP      = 22    ;   //unit:line

    

    //źŶ
    input               clk_in  ;
    input               rst_n   ;

    //źŶ
    output              dclk    ;
    output              hsync   ;
    output              vsync   ;
    //output              lcd_de  ;
    output              lcd_dis ;
    output[DATA_W-1:0]  data_r  ;
    output[DATA_W-1:0]  data_g  ;
    output[DATA_W-1:0]  data_b  ;

    //źreg
    wire                dclk    ;
    reg                 hsync   ;
    reg                 vsync   ;
    reg                 lcd_de  ;
    reg                 lcd_dis ;
    reg   [DATA_W-1:0]  data_r  ;
    reg   [DATA_W-1:0]  data_g  ;
    reg   [DATA_W-1:0]  data_b  ;

    reg [ 6:0]  cnt0     ;
    wire        add_cnt0 ;
    wire        end_cnt0 ;
    reg [ 3:0]  cnt1     ;
    wire        add_cnt1 ;
    wire        end_cnt1 ;
    wire        active_rec;
    wire[23:0]  data_rgb  ;

    //мźŶ

    /*********www.mdy-edu.com ƽ עͿʼ****************
    PLL,generate 40Mhz
    **********www.mdy-edu.com ƽ עͽ****************/
    wire    clk     ;
    pll_40m u_pll_40m(
	    .areset     (~rst_n ),
        .inclk0     (clk_in ),
	    .c0         (clk    )
    );
    
    /*********www.mdy-edu.com ƽ עͿʼ****************
    counter
    **********www.mdy-edu.com ƽ עͽ****************/
    //cnt_hs
    reg[10:0]   cnt_hs      ;
    wire        add_cnt_hs  ;
    wire        end_cnt_hs  ;
    always @(posedge clk or negedge rst_n)begin
        if(!rst_n)begin
            cnt_hs <= 0;
        end
        else if(add_cnt_hs)begin
            if(end_cnt_hs)
                cnt_hs <= 0;
            else
                cnt_hs <= cnt_hs + 1;
        end
    end

    assign add_cnt_hs = 1'b1;       
    assign end_cnt_hs = add_cnt_hs && cnt_hs== THB + THD + THFP - 1;     


    //cnt_vs
    reg[9:0]    cnt_vs      ;
    wire        add_cnt_vs  ;
    wire        end_cnt_vs  ;
    always @(posedge clk or negedge rst_n)begin
        if(!rst_n)begin
            cnt_vs <= 0;
        end
        else if(add_cnt_vs)begin
            if(end_cnt_vs)
                cnt_vs <= 0;
            else
                cnt_vs <= cnt_vs + 1;
        end
    end

    assign add_cnt_vs = end_cnt_hs;       
    assign end_cnt_vs = add_cnt_vs && cnt_vs== TVB + TVD + TVFP - 1;     



    always @(posedge clk or negedge rst_n) begin 
        if (rst_n==0) begin
            cnt0 <= 0; 
        end
        else if(add_cnt0) begin
            if(end_cnt0)
                cnt0 <= 0; 
            else
                cnt0 <= cnt0+1 ;
       end
    end
    assign add_cnt0 = end_cnt_vs;
    assign end_cnt0 = add_cnt0  && cnt0 == 60-1 ;



    always @(posedge clk or negedge rst_n) begin 
        if (rst_n==0) begin
            cnt1 <= 0; 
        end
        else if(add_cnt1) begin
            if(end_cnt1)
                cnt1 <= 0; 
            else
                cnt1 <= cnt1+1 ;
       end
    end
    assign add_cnt1 = end_cnt0;
    assign end_cnt1 = add_cnt1  && cnt1 == 8-1 ;



    /*********www.mdy-edu.com ƽ עͿʼ****************
    output
    dclk/hsync/vsync/lcd_de/data_r/data_g/data_b/lcd_dis
    **********www.mdy-edu.com ƽ עͽ****************/
    //dclk
    assign dclk = clk;

    //hsync
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            hsync <= 0;
        end
        else if(add_cnt_hs && cnt_hs==THPW-1)begin
            hsync <= 1;
        end
        else if(end_cnt_hs)begin
            hsync <= 0;
        end
    end

    //vsync
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            vsync <= 0;
        end
        else if(add_cnt_vs && cnt_vs==TVPW-1)begin
            vsync <= 1;
        end
        else if(end_cnt_vs)begin
            vsync <= 0;
        end
    end

    //lcd_de
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            lcd_de <= 0;
        end
        else if(active_area)begin
            lcd_de <= 0;
        end
        else begin
            lcd_de <= 0;
        end
    end

    //data_r
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            data_r <= 0;
        end
        else if(active_area)begin
            if(active_rec)
                data_r <= data_rgb[23:16];
            else
                data_r <= 8'h00;
        end
        else begin
            data_r <= 0;
        end
    end

    //data_g
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            data_g <= 0;
        end
        else if(active_area)begin
            if(active_rec)
                data_g <= data_rgb[15:8];
            else
                data_g <= 8'h00;

        end
        else begin
            data_g <= 0;
        end
    end

    //data_b
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            data_b <= 0;
        end
        else if(active_area)begin
            if(active_rec)
                data_b <= data_rgb[7:0];
            else
                data_b <= 8'h00;
        end
        else begin
            data_b <= 0;
        end
    end


    assign active_area = cnt_hs>=THB-1 && cnt_hs<THB+THD-1 && cnt_vs>=TVB-1 && cnt_vs<TVB+TVD-1;


    assign active_rec  = cnt_hs>=THB+cnt0*8-1 && cnt_hs<THB+320+cnt0*8-1 && cnt_vs>=TVB+cnt0*4-1 && cnt_vs<TVB+160+cnt0*4-1;

    assign data_rgb = 24'hffffff << (cnt1*3);


    //lcd_dis
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            lcd_dis <= 0;
        end
        else begin
            lcd_dis <= 1;
        end
    end




    endmodule

